#!/usr/bin/env python

import os, tempfile
try:
	from subprocess import Popen,PIPE
	def _runCommand(cmd, data):
		p = Popen([cmd], bufsize=1, stdin=PIPE, stdout=PIPE, stderr=PIPE,close_fds=True, shell=True)
		out, err = p.communicate(input=data)
		return (out, err)
except:
	from popen2 import Popen3
	def _runCommand(cmd, data):
		p = Popen3(cmd, 1)
		p.tochild.write(data)
		p.tochild.close()
		out = p.fromchild.read()
		p.fromchild.close()
		err = p.childerr.read()
		p.childerr.close()
		p.wait()
		return (out, err)

def decrypt(data, cert, privkey):
	msg, err = _runCommand("openssl smime -decrypt -inform PEM -recip %s -inkey %s" % \
							(cert,privkey), data)
	if err != '':
		raise Exception('openssl failed: %s' % err)
	return msg

def encrypt(data, cert):
	msg, err = _runCommand("openssl smime -encrypt -des3 -outform PEM %s" % cert, data)
	if err != '':
		raise Exception('openssl failed: %s' % err)
	return msg

def encodeFileB64(inp, out):
	_, err = _runCommand("openssl base64 -in %s -out %s" % (inp, out), "")
	if err != '':
		raise Exception('openssl failed: %s' % err)
	
def decodeFileB64(inp, out):
	_, err = _runCommand("openssl base64 -d -in %s -out %s" % (inp, out), "")
	if err != '':
		raise Exception('openssl failed: %s' % err)
			
def signFile(inp, out, cert, privkey):
	b64 = _mkstemp()
	try:
		encodeFileB64(inp, b64)
		_, err = _runCommand("openssl smime -sign -signer %s -inkey %s -in %s -out %s" % \
						 (cert, privkey, b64, out), "")
		if err != '':
			raise Exception('openssl failed: %s' % err)
	finally:
		try: os.remove(b64)
		except:	pass
	
def verifyFile(inp, out, certs):
	"""Input File to verify, Output File with verified message, Cert directory"""
	signer = _mkstemp()
	b64 = _mkstemp()
	try:
		_, err = _runCommand("openssl smime -verify -CApath %s -signer %s -out %s -in %s" % \
							 (certs, signer, b64, inp), "")
		if err != 'Verification successful\n':
			raise Exception('openssl failed: %s' % err)
		decodeFileB64(b64, out)
		signerCert = "".join(open(signer,'r').readlines())
	finally:
		try: os.remove(signer)
		except:	pass
		try: os.remove(b64)
		except:	pass
		
	return signerCert
	
def x509subject(cert):
	msg, err = _runCommand("openssl x509 -subject -noout", cert)
	if err != '':
		raise Exception('openssl failed: %s' % err)
	return msg.replace("subject= ","",1).strip()

def _mkdtemp(directory = "."):
	directory = os.path.abspath(directory)
	out, err = _runCommand('/bin/mktemp -d %s/.XXXXXX' % directory, "")
	if err != '':
		raise Exception('mktemp failed: %s' % err)
	return os.path.abspath(out.strip())
	
def _mkstemp():
	path = None
	try:
		fd, path = tempfile.mkstemp()
		os.close(fd)
	except:
		path = tempfile.mktemp()
	return path
 